@@ -2,7 +2,7 @@
  *
  * Murat's Ident Daemon
  * 
- * (C) 1998-2000 Murat Deligonul
+ * (C) 1998-2002 Murat Deligonul
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,16 +41,16 @@
  */
 
 #include "autoconf.h"
-    
+
 #include <unistd.h>
 #include <sys/socket.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-
+#include <arpa/inet.h>
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
-
+#include <stdarg.h>
 #include <sys/wait.h>
 #include <sys/un.h>
 #include <stdlib.h>
@@ -63,12 +63,15 @@
 #include <time.h>
 #include <ctype.h>
 #include <errno.h>
+#include <sys/time.h>
 
 #include "identd.h"
-#include "general.h"
 #include "defs.h"
+
 #include "lib_mdidentd.h"
 
+static int daemon(bool cr);
+
 static int unix_listen_fd, listen_fd;
 static bool foreground, auto_kill;  /* Some options */
 bool no_real;
@@ -81,16 +84,19 @@ static char ** identd_args;
 static int num_cmdargs;
 static int max_time;               /* How long fake ident requests will live */
     
-static linkedlist<conn> conns;      /* store the connections on port 113 */
-static linkedlist<conn> unix_conns; /* store connections to the domain socket */
+static list<conn> conns;      /* store the connections on port 113 */
+static list<conn> unix_conns; /* store connections to the domain socket */
 static ilist * table;               /* store all of the fake idents */
 
+int gettok(const char *, char *, unsigned long, char, int, bool = 0);
+int safe_strcpy(char *, const char *, unsigned long );
+int fdprintf(int fd, const char *, ...);
 
-const char *MDIDENTD_VERSION = "0.85.3";
+const char *MDIDENTD_VERSION = "0.85.5";
 
 inline void usage(void) 
 {
-    fprintf(stderr, "mdidentd v%s - Murat Deligonul (druglord@freelsd.org) - (C) 1998,1999\n"
+    fprintf(stderr, "mdidentd v%s - Murat Deligonul (druglord@freelsd.org) - (C) 1998-2002\n"
                    "\nUsage:\n"
                    "mdidentd [-f|-k|-h|-u] <identd> [options]\nWhere:\n"
                    "-u <userid>: Setuid to this user after binding sockets\n"
@@ -273,7 +279,7 @@ static int listen_unix(char const * path)
     }
     /* Must make it world writable if we expect to receive connections 
        on it */
-    fchmod(unix_listen_fd, 0666);
+    fchmod(unix_listen_fd, 0777);
     return 1;
 }
 
@@ -421,6 +427,7 @@ static int reply(conn * c)
                         p1, p2, ident);
                 if (auto_kill)
                     table->remove(p1, p2, ident);
+                close_conn(c);
                 return 1;
             }
         }
@@ -481,6 +488,10 @@ static int start_server(time_t max)
     if (!listen_unix(MDIDENTD_SOCK_PATH))
         return perror(MDIDENTD_SOCK_PATH), 1;
 
+    /* we already did this. but it doesn't work. i dunno why.
+     * i am messing up somwhere. dunno where. but we will do it again.
+     * and it will work. so there!! */
+    chmod(MDIDENTD_SOCK_PATH, 0777);
 #ifndef NOFORK
     if (!foreground)    
     {
@@ -526,14 +537,17 @@ static int start_server(time_t max)
         FD_ZERO(&fds);
         FD_SET(listen_fd, &fds);
         FD_SET(unix_listen_fd, &fds);
-        
-        for (; (c = i.get()); i++)
-        {
+
+       	while (i.has_next())
+       	{
+			c = i.next();
             FD_SET(c->fd, &fds);
             hf = (hf > c->fd) ? hf : c->fd;
         }
-        for (; (c = i2.get()); i2++)
+
+        while (i2.has_next())
         {
+        	c = i2.next();
             FD_SET(c->fd, &fds);
             hf = (hf > c->fd) ? hf : c->fd;
         }
@@ -573,13 +587,13 @@ static int start_server(time_t max)
         i.set(0);
         i2.set(0);
         time(&curtime);
-        for (; (c = i.get()); i++)
+        while (i.has_next())
         {
+        	c = i.next();
             if (c->fd != listen_fd && FD_ISSET(c->fd, &fds) && service(c) == -1)
             {
-                i++;
                 delete c;
-                conns.remove(c);
+                i.remove();
                 num_conns--;
             } /* Check for idlers while we're here */
             else if (c->fd != listen_fd && curtime - c->time >= MAX_IDENT_WAIT_TIME)
@@ -587,31 +601,29 @@ static int start_server(time_t max)
                 struct sockaddr_in sin;
                 socklen_t size = sizeof(sin);
                 getpeername(c->fd, (struct sockaddr *) &sin, &size);
-                i++;
                 syslog(LOG_INFO, "Killing idling connection from %s", inet_ntoa(sin.sin_addr));
                 close_conn(c);
                 delete c;
-                conns.remove(c);
+                i.remove();
                 num_conns--;
             }
         }
 
-        for (; (c = i2.get()); i2++)
+        while (i2.has_next())
         {
+        	c = i2.next();
             if (c->fd != unix_listen_fd && FD_ISSET(c->fd, &fds) &&  service_unix(c) == -1)
             {
-                i2++;
                 delete c;
-                unix_conns.remove(c);
+                i2.remove();
                 num_unix--;
             } /* Check for idlers while we're here */
             else if (c->fd != unix_listen_fd && curtime - c->time >= MAX_UNIX_WAIT_TIME)
             {
-                i++;
                 close_conn(c);
                 syslog(LOG_INFO, "Killing idling connection on domain socket");
                 delete c;
-                unix_conns.remove(c);
+                i2.remove();
                 num_unix--;
             }
         }
@@ -635,7 +647,6 @@ int main(int argc, char **argv)
 
 void sighandler(int sig)
 {
-    
     switch (sig)
     {
     case SIGCHLD:
@@ -667,3 +678,108 @@ void sighandler(int sig)
             
     }
 }
+
+/*
+ * Return pid of child to parent.
+ * 0 to child.
+ * -1 on error.
+ */
+static int daemon(bool cr)
+{
+    pid_t xx;
+    switch ((xx = fork()))
+    {
+    case 0:
+        /* Child */
+        setsid();
+        close(STDIN_FILENO);
+        close(STDOUT_FILENO);
+        close(STDERR_FILENO);
+        if (cr)
+            chroot("/");
+        return 0;
+
+    case -1:
+        return -1;
+
+    default:
+        return xx;
+
+    }
+}
+
+
+/*
+ *  Gets which'th token, seperated by sep
+ *  return 1 on success 0 on failure.
+ *  Last argument indicates whether.
+ *
+ *  Null argument can be passed as buffer to indicate you only
+ *  want to check the existance of a token.
+ */
+int gettok(const char *p, char *buffer, unsigned long maxsize,
+           char sep, int which, bool get_rest)
+{
+    int numfound = 1;
+    /* remove leading seps */
+    while (*p == sep)
+        p++;
+    do {
+        if (numfound == which && *p)
+        {
+            char * next = strchr(p, sep);
+            if ((get_rest && buffer) || (!next /* && *(p+1) */) )
+            {
+                if (buffer)
+                    safe_strcpy(buffer, p, maxsize);
+                return 1;
+            }
+            else if (next)
+            {
+                if (buffer)
+                {
+                    u_long bytes2copy = (next - p) + 1; /* how many bytes must be copied, including
+                                            NULL character */
+                    if (bytes2copy >= maxsize)
+                        bytes2copy = maxsize;
+                    strncpy(buffer, p, bytes2copy);
+                    buffer[bytes2copy - 1] = 0;
+                }
+                return 1;
+            }
+            return 0;
+        }
+        /* check if this letter == sep, and the next letter is not sep */
+        if (*p == sep && *(p+1) != sep)
+            numfound++;
+    } while (*++p);
+    /* failed */
+    return 0;
+}
+
+
+/*
+ *  Copies src to dest, pretends dest is maxsize long.
+ *  will truncate by 1 if needs room to add \0 (bad?)
+ */
+int safe_strcpy(char *dest, const char *src, unsigned long maxsize)
+{
+    unsigned long len = strlen(src);
+
+    if (len >=maxsize)
+        len = maxsize - 1;
+    memcpy(dest, src, len);
+    dest[len] = 0;
+    return 1;
+}
+
+int fdprintf(int fd, const char *format, ...)
+{
+    static char __mbuffer[1024];
+    __mbuffer[0] = 0;
+    va_list ap;
+    va_start(ap, format);
+    int len = vsnprintf(__mbuffer, sizeof(__mbuffer), format, ap);
+    va_end(ap);
+    return (write(fd, __mbuffer, len));
+}
