@@ -1,7 +1,7 @@
 /*
  * logfile.cpp
  *
- * (c) 1999, 2000 Murat Deligonul
+ * (c) 1999-2002 Murat Deligonul
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,28 +19,8 @@
  *
  */
  
-        
-/*
- * TODO:
- * - Size variables updated incorrectly if not logging seperately (still true?)
- */
- 
-                
-/*
- * some extended info is encryped into the log file name; the format is:
- * pid username userid detachedpasswordinsomeweirdencrpytion private channel [nothing].log
- *
- * We use spaces for seperators. The main reason for this is to keep me sane.
- * It is hard enough manipulating C style strings, it is a total nightmare when
- * the seperator character is not unique. I.e., using, say, '-' as the seperator
- * sounds cool but really it is a legal character to use in Nicknames and passwords,
- * and will therefore be a mess to parse. 
- *
- * We use spaces. Murat is happy.
- */
-#include "autoconf.h"
 
-                   
+#include "autoconf.h"
 #include <sys/types.h>
 #include <unistd.h>
 #include <sys/stat.h>
@@ -48,18 +28,17 @@
 #include <stdlib.h>
 #include <ctype.h>
 #include <dirent.h>
-
+#include "debug.h"
 #include "linkedlist.h"                     
 #include "ezbounce.h"                            
 #include "general.h"
 #include "logfile.h"
-#include "objset.h"
 #include "server.h"
 
 static char * cheesy_encrypt(char *);
 static char * cheesy_decrypt(char *);
 
-linkedlist<char> logfile::active_logs;
+list<char> logfile::active_logs;
 
 /*
  * open()'s a new logfile, ensuring that is unique and not locked by anyone
@@ -68,7 +47,7 @@ linkedlist<char> logfile::active_logs;
 /* static */ int logfile::mkname(const char * basedir, const char * owner, int id, 
                                   const char * pw, const char * type, char * buff)
 {
-    static const char *extra[] = { "", "1.", "2.", "3.", "4.", "5.", "6." };
+    static const char * const extra[] = { "", "1.", "2.", "3.", "4.", "5.", "6." };
     int c = 0, fd = -1;
     while (1)
     {
@@ -98,7 +77,7 @@ linkedlist<char> logfile::active_logs;
 logfile::logfile(const char * basedir, const char * owner, int id, int options, int maxsize, const char * password, int * result)
 {
     char filebuff[PATH_MAX + 1];
-    char * pw = my_strdup(password);
+    char * pw = (password ?  my_strdup(password) : my_strdup("none"));
     
     pw = cheesy_encrypt(pw);
     
@@ -223,40 +202,47 @@ logfile::~logfile(void)
 }
 
 
-/* Written number of bytes written
- * 0 if nothing could be */
-int logfile::write(const char * raw)
+/*
+ * The interface to the log files -- we do the parsing
+ * elsewhere.. Arguments:
+ *
+ *  flags           - combination of options to specify the event
+ * source           - who it's coming from (in a full ircaddr structure)
+ * target           - who it's to
+ * text             - chat text, or part & quit part messages
+ *                    this is not const; we need to be able to modify it
+ *                    so we can get rid of the color codes and stuff
+ * extra            - for anything that don't fit; for example in KICK, who got kicked
+ *                    or in CTCPs, the CTCP itself, like CLIENTINFO or VERSION or whatever
+ */
+
+int logfile::write(int flags, const __ircaddr * source, const char * target, char * text, const char * extra)
 {
-    char source[ADDRESS_LENGTH + 1];
-    char target[CHANNEL_LENGTH + 1];
-    char target2[NICKNAME_LENGTH + 1] = "";
-    char text[TEXT_LENGTH + 1] = "";
-    char extra[CHANNEL_LENGTH + 5] = ":";
     int target_fd = fd1;
     unsigned long * size = &size1;
     int written = 0;
 
-    int flags = parse(raw, source, target, target2, text);
-    if (flags & OTHER)
-        return 0;
-
-    if (flags & PRIVATE)
+    if (flags & EVENT_PRIVATE)
     {
         if (!(options & LOG_PRIVATE))
             return 0;
-        extra[0] = 0;
-        target_fd = fd2;
-        size = &size2;
+        if (!log2)
+        {
+            size = &size1;
+            target_fd = fd1;
+        } else {
+            size = &size2;
+            target_fd = fd2;
+        }
     }
 
-    /* Check file size now */
+    /* Enforce the size limits now */
     if ((maxsize) && (*size > (unsigned) maxsize))
     {
         fdprintf(target_fd, "*** [NOTE] Logfile size limit of %d bytes exceeded; stopping log ***\n", maxsize);
         fdprintf(target_fd, "******************************************************\n"
                         "ezbounce log file stopped at %s\n"
-                        "******************************************************\n", 
-                 timestamp());
+                        "******************************************************\n", timestamp());
         close(target_fd);
         if (target_fd == fd1)
             fd1 = -1;
@@ -264,151 +250,76 @@ int logfile::write(const char * raw)
             fd2 = -1;
         return 0;
     }
-        
-
-    else if (flags & PUBLIC)
+    else if (flags & EVENT_PUBLIC)
     {
-        /* Check if we really need to log (though this is redundant, checks
-         * happen elsewhere */
+        /* Check if we really need to log */
         if (!(options & LOG_CHANNEL))
             return 0;
-        /* Include full address in public channel messages ? */
-        if (!(options & LOG_FULL_ADDRS) && (flags & MESSAGE))
-        {
-            /* If not, truncate after the nickname */
-            char * pos = strchr(source, '!');
-            if (pos)
-                *pos = 0;
-        }
-        strcat(extra, target);
     }
-                             
-    strip_burc_codes(text);
+
+    if (text)
+        strip_burc_codes(text);
 
     if (options & LOG_TIMESTAMP)
         written += fdprintf(target_fd, "%s ", timestamp(1));
-    if (flags & NOTICE)
-        written += fdprintf(target_fd, "-%s%s- %s\n", no_leading(source), extra, no_leading(text));
-    else if (flags & JOIN)
-        written += fdprintf(target_fd, "*** %s has JOINed %s\n", no_leading(source), no_leading(target));
-    else if (flags & PART)
-        written += fdprintf(target_fd, "*** %s has PARTed %s (%s)\n", no_leading(source), no_leading(target), 
-                           no_leading(text));
-    else if (flags & TOPIC)
-        written += fdprintf(target_fd, "*** %s changes %s TOPIC to: %s\n", no_leading(source), no_leading(target), 
-                           no_leading(text));
-    else if (flags & KICK)
-        written += fdprintf(target_fd, "*** %s has KICKed %s from %s (%s)\n", no_leading(source), no_leading(target2), 
-                           target, no_leading(text));
-    else if (flags & MODE)
-        written += fdprintf(target_fd, "*** %s changes %s MODE to: %s\n", no_leading(source), target, no_leading(text));
-    else if (flags & CTCP)                            
+    if (flags & EVENT_NOTICE)
+        written += fdprintf(target_fd, "-%s!%s@%s:%s- %s\n", source->nick,
+                                    source->ident, source->host, target, text);
+    else if (flags & EVENT_JOIN)
+        written += fdprintf(target_fd, "*** %s (%s@%s) has JOINed %s\n", source->nick,
+                                    source->ident, source->host, target);
+    else if (flags & EVENT_PART)
+        written += fdprintf(target_fd, "*** %s (%s@%s) has PARTed %s (%s)\n", source->nick,
+                                    source->ident, source->host, target, text);
+    else if (flags & EVENT_TOPIC)
+        written += fdprintf(target_fd, "*** %s changes %s TOPIC to: %s\n", source->nick, target, text);
+    else if (flags & EVENT_KICK)
+        written += fdprintf(target_fd, "*** %s (%s@%s) has KICKed %s from %s (%s)\n", source->nick, source->ident,
+                                    source->host, extra, target, text);
+    else if (flags & EVENT_MODE)
+        written += fdprintf(target_fd, "*** %s changes %s MODE to: %s\n", source->nick, target, text);
+    else if (flags & EVENT_QUIT)
+        written += fdprintf(target_fd, "*** %s (%s@%s) has QUIT (%s)\n", source->nick, source->ident,
+                                                                    source->host, text);
+    else if (flags & EVENT_NICK)
+        written += fdprintf(target_fd, "*** %s changes NICK to %s\n", source->nick, target);
+    else if (flags & EVENT_CTCP)
     {
         /* Remove ending 001 char */
-        char * dummy = &text[strlen(text)] - 1;
-        if (*dummy == '\001')
-            *dummy = 0;
+        if (text)
+        {
+            char * dummy = &text[strlen(text)] - 1;
+            if (*dummy == '\001')
+                *dummy = 0;
+        }
 
-        /* Check for ACTION ctcps and do nicer formatting */
-        char tmp[10];
-        gettok(no_leading(text) + 1, tmp, sizeof(tmp), ' ', 1);
-        if (strcmp("ACTION", tmp) == 0)
-            written += fdprintf(target_fd, "* %s%s %s\n", no_leading(source), extra, no_leading(text) + 1 + 7);
+        if (flags & EVENT_PUBLIC)
+            written += fdprintf(target_fd, "[%s] ", target);
+
+        if (strcmp(extra, "ACTION") == 0)
+        {
+            if ((options & LOG_FULL_ADDRS) || (flags & EVENT_PRIVATE))
+                written += fdprintf(target_fd, "* %s!%s@%s %s\n", source->nick, source->ident, source->host, extra);
+            else
+                written += fdprintf(target_fd, "* %s %s\n", source->nick, extra);
+        }
         else
-            written += fdprintf(target_fd, "*** CTCP %s from %s to %s\n", no_leading(text) + 1, no_leading(source),
-                                        target);
-    } else
-        written += fdprintf(target_fd, "<%s%s> %s\n", no_leading(source), extra, no_leading(text));
-    
+            written += fdprintf(target_fd, "*** CTCP %s [%s] from %s (%s@%s) to %s\n", extra, text, source->nick,
+                                        source->ident, source->host, target);
+    } else {
+        if (flags & EVENT_PUBLIC)
+            written += fdprintf(target_fd, "[%s] ", target);
+
+        if ((options & LOG_FULL_ADDRS) || (flags & EVENT_PRIVATE))
+           written += fdprintf(target_fd, "<%s!%s@%s> %s\n", source->nick, source->ident, source->host, text);
+        else
+           written += fdprintf(target_fd, "<%s> %s\n", source->nick, text);
+    }
     if (written >= 0)
         *size += (unsigned long) written;
     return written;
 }
 
-
-/*
- * Split the message up into little chunks
- * Return WTF it is!
- */
-int logfile::parse(const char * raw, char * source, char * target, char * target2, char * text)
-{
-    /* Valid stuff handled is in the form:
-     * :[source] [PRIVMSG/NOTICE/JOIN/PART/TOPIC/MODE] [target] :text 
-     * :[source] KICK [target] [target2] :reason
-     */
-
-    int flags = 0;
-    char type[10];
-    
-    if (!gettok(raw, type, sizeof(type),' ', 2))
-        return OTHER;
-
-    if (strcmp(type, "PRIVMSG") == 0)
-        flags |= MESSAGE; 
-    else if (strcmp(type, "NOTICE") == 0)
-        flags |= NOTICE;
-    else if (options & LOG_CHANNEL)
-    {
-        if (strcmp(type, "JOIN") == 0)
-            flags |= JOIN;
-        else if (strcmp(type, "PART") == 0)
-            flags |= PART;
-        else if (strcmp(type, "TOPIC") == 0)
-            flags |= TOPIC;
-        else if (strcmp(type, "MODE") == 0)
-            flags |= MODE;
-        else if (strcmp(type, "KICK") == 0)
-        {
-            flags |= KICK;
-            /* Get target 2 -- that is, the person being kicked, as well 
-            as the reason */
-            if (!gettok(raw, target2, NICKNAME_LENGTH, ' ', 4) || 
-                !gettok(raw, text, TEXT_LENGTH, ' ', 5, 1))
-                return OTHER;
-            gettok(raw, target, CHANNEL_LENGTH , ' ', 3);
-            gettok(raw, source, ADDRESS_LENGTH, ' ', 1);
-            return flags;
-        }
-        else
-            return OTHER;
-    }
-    else
-        return OTHER;
-
-    /* Get the target */
-    if (!gettok(raw, target, CHANNEL_LENGTH , ' ', 3))
-        return OTHER;
-
-    if ((flags & NOTICE) || (flags & MESSAGE))
-    {
-        switch (target[0])
-        {
-        case '#':
-        case '+':       
-        case '&':
-        case '!':               /* new IRCNET stuff? */
-            if (!(options & LOG_CHANNEL))
-                return OTHER;
-            flags |= PUBLIC;
-            break;
-        default:
-            if (!(options & LOG_PRIVATE))
-                return OTHER;
-            flags |= PRIVATE;
-            flags &= ~PUBLIC;
-        }
-    }
-
-    gettok(raw, text, TEXT_LENGTH, ' ', 4,1);
-    gettok(raw, source, ADDRESS_LENGTH, ' ', 1);
-    
-    if ((flags & MESSAGE) && (text[0] == '\001' || text[1] == '\001'))
-        flags |= CTCP;
-    
-    return flags;
-}
-
-    
 /*
  * Convert character log control codes to logfile class
  * flags. Valid codes:
@@ -504,7 +415,7 @@ int logfile::dump(const char * raw)
  * f[0] will be all OR channel logs
  * f[1] will be NULL OR private logs
  */
-int logfile::get_filenames(const char *f[2])
+int logfile::get_filenames(char *f[2])
 {
     f[0] = my_strdup(log1);
     f[1] = my_strdup(log2);
@@ -569,14 +480,14 @@ int logfile::strip_burc_codes(char * text)
         *n++ = *text++;
     }
     *n = 0;
-    strcpy(orig, origN); 
+    strcpy(orig, origN);
+    delete[] origN;
 
     /* there's gotta be a better way.. */
     return 1;
 }
     
-
-
+/* i don't know anything about encryption */
 static char * cheesy_encrypt(char * text)
 {
     unsigned int i;
@@ -632,21 +543,23 @@ struct logfile_ent {
  * Arguments: 
  *      basedir - where to find logs
  *      nickname, password, id, self explanatatory
- *      logfiles[2] - holds names of users active log files AND will be used
- *      to store the new ones
  */
-/* static */ int logfile::find_log_files(const char * basedir, const char * /* nickname */, 
-                                         const char * password, int id, obj_set<char> * plist, int max)
+/* static */ int logfile::find_log_files(const char * basedir, const char * username,
+                                         const char * password, int id, list<char> * plist, int max)
 {
     DIR *dir = opendir(basedir);
     struct dirent *d = 0;
     int num_found = 0;
     static pid_t mypid = getpid();
 
+    assert(username);
+    if (!password)
+        password = "none";
+
     if (!dir)
         return -1;
     
-    obj_set<struct logfile_ent> list;
+    list<struct logfile_ent> list;
     struct logfile_ent * lt = 0;
 
     errno = 0;
@@ -669,9 +582,14 @@ struct logfile_ent {
         if (args < 4) 
             continue;
 
-        /* Compare the id and password -- 
-         * NOTE: right now we don't check for nickname */
-        if (lt->uid == id && !strcasecmp(password, cheesy_decrypt(lt->file_pw)))
+        /*
+         * Check that the username matches
+         *       that the passwords match (specify "none" if no pass!!)
+         *       and that the id's match, if a valid id was provided
+         */
+        if (!strcasecmp(username, lt->file_nick)
+             && !strcasecmp(password, cheesy_decrypt(lt->file_pw))
+             && (id < 0 || id == lt->uid))
         {
             num_found++;
             lt->filename = new char[strlen(basedir) + strlen(d->d_name) + 5];
@@ -688,15 +606,13 @@ struct logfile_ent {
         delete lt;
         lt = 0;
     }
-    
-
     if (num_found)
     {
         int cur = 0;
         /* Go through the whole list, favor ones with current PID */
         for (int i = 0; i < num_found && cur < max; i++)
         {
-            lt = list.objs[i];
+            lt = list.get(i);
             if (lt->pid == mypid)
             {
                 cur++;
@@ -707,7 +623,7 @@ struct logfile_ent {
         /* Fill in other matches.    */
         for (int i = 0; i < num_found && cur < max; i++)
         {
-            lt = list.objs[i];
+            lt = list.get(i);
             if (lt->pid != mypid)
             {
                 cur++;
@@ -718,12 +634,7 @@ struct logfile_ent {
         lt = 0;
         
         /* We got the filenames now, so clean up the mess */
-        for (int i = 0; i < num_found; i++)
-        {
-            delete[] list.objs[i]->filename;
-            delete list.objs[i];
-        }
-
+        destroy_list(&list, 1);
         return num_found;
     }    
     return 0;
@@ -745,31 +656,19 @@ struct logfile_ent {
 
 /* static */ int logfile::release(const char * file)
 {
-    list_iterator<char> i (&active_logs);
-    char * f;
     file = nopath(file);
     DEBUG("logfile::release() called on %s\n", file);
-    for (; (f = i.get()); ++i)
-    {
-        if (strcmp(file, f) == 0)
-        {
-            active_logs.remove(f);
-            delete[] f;
-            return 1;
-        }
-    }
-    return 0;
+    return strlist_remove(&active_logs, file);
 }
 
 
 /* static */ int logfile::is_locked(const char * file)
 {
     list_iterator<char> i (&active_logs);
-    char * f;
     file = nopath(file);
     DEBUG("Lock Check on file %s\n", file);
-    for (; (f = i.get()); ++i)
-        if (strcmp(file, f) == 0)
+    while (i.has_next())
+        if (strcmp(file, i.next()) == 0)
             return 1;
 
     return 0;
@@ -786,8 +685,8 @@ const char * logfile::fixup_logname(const char * filename)
     }
     
     filename = nopath(filename);
-    if (!fixup_buff)
-        fixup_buff = new char[strlen(filename) + 20];
+    int len = strlen(filename) + 20;
+    fixup_buff = new char[len];
     
     char nick[NICKNAME_LENGTH], id[10], suffix[20];
     time_t blah = ircproxy_time();
@@ -797,6 +696,6 @@ const char * logfile::fixup_logname(const char * filename)
     gettok(filename, id, sizeof(id), ' ', 3);
     gettok(filename, suffix, sizeof(suffix), ' ', 5);
 
-    sprintf(fixup_buff, "%d%02d%02d-%s-[%s]-%s", 1900 + t->tm_year, t->tm_mon + 1, t->tm_mday, nick, id, suffix);
+    snprintf(fixup_buff, len, "%d%02d%02d-%s-[%s]-%s", 1900 + t->tm_year, t->tm_mon + 1, t->tm_mday, nick, id, suffix);
     return fixup_buff;
 }
